//+------------------------------------------------------------------+
//|                                                   CArrayRing.mqh |
//|                               Copyright 2012, Konstantin Gruzdev |
//|                            https://login.mql5.com/ru/users/Lizar |
//|                                             Revision 03 Dec 2012 |
//+------------------------------------------------------------------+
#property copyright   "Copyright 2012, Konstantin Gruzdev"
#property link        "https://login.mql5.com/ru/users/Lizar"

//+------------------------------------------------------------------+
//| Class CArrayRing                                                 |
//| Appointment:       |
//|     ,         |
//|         .  ,    |
//|                |
//|    .                                                   |
//| Link: http://www.mql5.com/ru/code/1340                           |
//| Remark:  ,       |
//|        .                               |
//+------------------------------------------------------------------+
class CArrayRing
  {
private:
   double            m_data[];         //   
   int               m_size;           //  
   int               m_last_pos;       //    
   double            m_filling;        // ,   

public:
                     CArrayRing();
                    ~CArrayRing()                  { ArrayFree(m_data);                         }
   //---   :
   bool              Init(int size, double volue=EMPTY_VALUE);             
   //---    :
   int               Size()                        { return m_size-1;                           }
   //---     :
   bool              Resize(const int size);
   //---      :
   void              Add(const double element); 
   //---       :
   double            At(const int index) const;
   double operator   [](const int index) const     { return(At(index));                         }
   //---        :
   double            Last() const                  { return(m_data[m_last_pos]);                } 
   //---       :
   void              Last(const double element)    { m_data[m_last_pos]=element;                }  
   //---       :
   bool              Update(const double element,const int index=0);  
  };
//+------------------------------------------------------------------+
//| .                                                     |
//+------------------------------------------------------------------+
CArrayRing::CArrayRing()
  {
   m_last_pos=0;                          //   
   m_filling=EMPTY_VALUE;                 //    
   m_size=ArraySize(m_data);              //    
  }

//+------------------------------------------------------------------+
//|   .                                      |
//+------------------------------------------------------------------+
bool CArrayRing::Init(int size, double volue=EMPTY_VALUE)
  {
   m_last_pos=0;                          //   
   m_filling=volue;                       //    
   m_size=ArraySize(m_data);              //      
   bool result=Resize(size);              //     
   ArrayFill(m_data,0,m_size,m_filling);  //     
   return(result);                  
  }  

//+------------------------------------------------------------------+
//|    .                              |
//+------------------------------------------------------------------+
bool CArrayRing::Resize(const int new_size)
  {
//--- check
   if(new_size<0) return(false);
//---   :
   if(new_size>m_size)
     {
      int set_size=ArrayResize(m_data,new_size);
      if(set_size<0) return(false);
      //---      :
      if(set_size>m_size)
        {
         for(int i=m_size-1,j=set_size-1;i>m_last_pos;i--,j--)
           {
            m_data[j]=m_data[i];
            m_data[i]=m_filling;
           }
        }
      m_size=set_size;
      //--- result:
      return(true);      
     }
//---   :
   //---     :
   if(new_size>m_last_pos+1)
      for(int i=m_size-1,j=new_size-1;j>m_last_pos;i--,j--) m_data[j]=m_data[i];
   else
     {
      for(int i=m_last_pos+1-new_size,j=0;i<=m_last_pos;i++,j++) m_data[j]=m_data[i];    
      m_last_pos=new_size-1;
     }
   //---  :
   m_size=new_size;
   ArrayResize(m_data,new_size);
//--- result:
   return(true);
  }
  
//+------------------------------------------------------------------+
//|     .                              |
//+------------------------------------------------------------------+
void CArrayRing::Add(const double element)
  {
   m_last_pos=++m_last_pos%m_size;
   m_data[m_last_pos]=element;
  } 
  
//+------------------------------------------------------------------+
//|     .                           |
//+------------------------------------------------------------------+
double CArrayRing::At(const int index) const
  {
//---   :
   if((index/m_size)==0) 
//---      :
      return(m_data[(m_size+m_last_pos-index)%m_size]);
//---    :
   return(DBL_MAX);   
  }  

//+------------------------------------------------------------------+
//|      .              |
//+------------------------------------------------------------------+
bool CArrayRing::Update(const double element,const int index=0)
  {
//---   :
   if((index/m_size)==0) 
     {
//--- update
      m_data[(m_size+m_last_pos-index)%m_size]=element;
//--- successful
      return(true);
     }
//---    :
   return(false);   
  }

    
